import { _ } from '@eslint-react/eff';
import * as tseslint from '@typescript-eslint/utils/ts-eslint';
import { ReportDescriptor } from '@typescript-eslint/utils/ts-eslint';
import { CompilerOptions, JsxEmit } from 'typescript';
import * as valibot from 'valibot';
import { InferOutput } from 'valibot';

/**
 * Rule severity.
 * @since 0.0.1
 */
type RuleSeverity = "error" | "off" | "warn";
/**
 * Rule declaration.
 * @internal
 * @since 0.0.1
 */
type RuleDeclaration = [RuleSeverity, Record<string, unknown>?] | RuleSeverity;
/**
 * Rule config preset.
 * @since 0.0.1
 */
type RulePreset = Record<string, RuleDeclaration>;
/**
 * Rule context.
 * @since 0.0.1
 */
type RuleContext<MessageIds extends string = string, Options extends readonly unknown[] = readonly unknown[]> = tseslint.RuleContext<MessageIds, Options>;
/**
 * Rule feature.
 * @since 1.20.0
 */
type RuleFeature = "CFG" | "DBG" | "FIX" | "MOD" | "TSC" | "EXP";
/**
 * Creates a report function that can conditionally report a descriptor.
 * @param context - The context of the rule
 * @returns A function that takes a descriptor and reports it if it's not null or undefined
 */
declare function createReport<MessageID extends string>(context: RuleContext): (descriptor: _ | null | ReportDescriptor<MessageID>) => void;

type JsxConfig = Pick<CompilerOptions, "reactNamespace" | "jsx" | "jsxFactory" | "jsxFragmentFactory" | "jsxImportSource">;
/**
 * Create a JsxConfig object
 * @returns JsxConfig
 */
declare function make$1(): JsxConfig;
/**
 * Get JsxConfig from RuleContext
 * @param context The RuleContext
 * @returns JsxConfig
 */
declare function getFromContext$1(context: RuleContext): {
    jsx: JsxEmit;
    jsxFactory: string;
    jsxFragmentFactory: string;
    jsxImportSource: string;
    reactNamespace: string;
};
/**
 * Get JsxConfig from annotation
 * @param context The RuleContext
 * @returns JsxConfig
 */
declare function getFromAnnotation(context: RuleContext): JsxConfig;

type index$1_JsxConfig = JsxConfig;
declare const index$1_getFromAnnotation: typeof getFromAnnotation;
declare namespace index$1 {
  export { type index$1_JsxConfig as JsxConfig, index$1_getFromAnnotation as getFromAnnotation, getFromContext$1 as getFromContext, make$1 as make };
}

/**
 * @internal
 */
declare const LanguagePreferenceSchema: valibot.ObjectSchema<{
    readonly indentStyle: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"tab", undefined>, valibot.LiteralSchema<"space", undefined>], undefined>, "space">;
    readonly indentWidth: valibot.OptionalSchema<valibot.NumberSchema<undefined>, 2>;
    readonly quoteStyle: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"single", undefined>, valibot.LiteralSchema<"double", undefined>], undefined>, "single">;
    readonly semicolons: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"always", undefined>, valibot.LiteralSchema<"asNeeded", undefined>], undefined>, "always">;
    readonly trailingCommas: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"all", undefined>, valibot.LiteralSchema<"es5", undefined>, valibot.LiteralSchema<"none", undefined>], undefined>, "all">;
    readonly jsxQuoteStyle: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"single", undefined>, valibot.LiteralSchema<"double", undefined>], undefined>, "double">;
}, undefined>;

/**
 * @internal
 */
type LanguagePreference = InferOutput<typeof LanguagePreferenceSchema>;
/**
 * The default language preference.
 */
declare const DEFAULT_LANGUAGE_PREFERENCE: {
    readonly indentStyle: "space";
    readonly indentWidth: 2;
    readonly jsxQuoteStyle: "double";
    readonly quoteStyle: "single";
    readonly semicolons: "always";
    readonly trailingCommas: "all";
};
/**
 * Get a copy of the default LanguagePreference.
 */
declare function make(): LanguagePreference;
declare function getFromContext(): void;
declare module "@typescript-eslint/utils/ts-eslint" {
    interface SharedConfigurationSettings {
    }
}

declare const index_DEFAULT_LANGUAGE_PREFERENCE: typeof DEFAULT_LANGUAGE_PREFERENCE;
type index_LanguagePreference = LanguagePreference;
declare const index_LanguagePreferenceSchema: typeof LanguagePreferenceSchema;
declare const index_getFromContext: typeof getFromContext;
declare const index_make: typeof make;
declare namespace index {
  export { index_DEFAULT_LANGUAGE_PREFERENCE as DEFAULT_LANGUAGE_PREFERENCE, type index_LanguagePreference as LanguagePreference, index_LanguagePreferenceSchema as LanguagePreferenceSchema, index_getFromContext as getFromContext, index_make as make };
}

/**
 * Regular expression for matching a TypeScript file extension.
 */
declare const RE_TS_EXT: RegExp;
/**
 * Regular expression for matching a JavaScript file extension.
 */
declare const RE_JS_EXT: RegExp;
/**
 * Regular expression for matching a PascalCase string.
 */
declare const RE_PASCAL_CASE: RegExp;
/**
 * Regular expression for matching a camelCase string.
 */
declare const RE_CAMEL_CASE: RegExp;
/**
 * Regular expression for matching a kebab-case string.
 */
declare const RE_KEBAB_CASE: RegExp;
/**
 * Regular expression for matching a snake_case string.
 */
declare const RE_SNAKE_CASE: RegExp;
/**
 * Regular expression for matching a CONSTANT_CASE string.
 */
declare const RE_CONSTANT_CASE: RegExp;
declare const RE_JAVASCRIPT_PROTOCOL: RegExp;
/**
 * Regular expression for matching a valid JavaScript identifier.
 */
declare const RE_JS_IDENTIFIER: RegExp;
/**
 * Regular expression for matching a RegExp string.
 */
declare const RE_REGEXP_STR: RegExp;
/**
 * Regular expression for matching a `@jsx` annotation comment.
 */
declare const RE_ANNOTATION_JSX: RegExp;
/**
 * Regular expression for matching a `@jsxFrag` annotation comment.
 */
declare const RE_ANNOTATION_JSX_FRAG: RegExp;
/**
 * Regular expression for matching a `@jsxRuntime` annotation comment.
 */
declare const RE_ANNOTATION_JSX_RUNTIME: RegExp;
/**
 * Regular expression for matching a `@jsxImportSource` annotation comment.
 */
declare const RE_ANNOTATION_JSX_IMPORT_SOURCE: RegExp;
/**
 * Convert a string to the `RegExp`.
 * Normal strings (e.g. `"foo"`) is converted to `/^foo$/` of `RegExp`.
 * Strings like `"/^foo/i"` are converted to `/^foo/i` of `RegExp`.
 * @see https://github.com/sveltejs/eslint-plugin-svelte/blob/main/packages/eslint-plugin-svelte/src/utils/regexp.ts
 * @param string The string to convert.
 * @returns Returns the `RegExp`.
 */
declare function toRegExp(string: string): {
    test(s: string): boolean;
};
/**
 * Checks whether given string is regexp string
 * @param string The string to check
 * @returns boolean
 */
declare function isRegExp(string: string): boolean;

export { index$1 as JsxConfig, index as LanguagePreference, RE_ANNOTATION_JSX, RE_ANNOTATION_JSX_FRAG, RE_ANNOTATION_JSX_IMPORT_SOURCE, RE_ANNOTATION_JSX_RUNTIME, RE_CAMEL_CASE, RE_CONSTANT_CASE, RE_JAVASCRIPT_PROTOCOL, RE_JS_EXT, RE_JS_IDENTIFIER, RE_KEBAB_CASE, RE_PASCAL_CASE, RE_REGEXP_STR, RE_SNAKE_CASE, RE_TS_EXT, type RuleContext, type RuleDeclaration, type RuleFeature, type RulePreset, type RuleSeverity, createReport, isRegExp, toRegExp };
